<?php

/**
 * Sipgate API base class
 * @author Cedric May <may@riverlabs.de>
 * @link https://github.com/riverlabs/sipgate
 * @link https://www.live.sipgate.de/api/rest
 * @example CURRENT_FOLDER/examples/samples.php
 */
 
namespace Sipgate 
{
	
	class Main
	{
		private static $username;
		private static $password;
		
		const API_VERSION = '2.37.0';
		
		public function login($username, $password) 
		{
			self::$username = $username;
			self::$password = $password;
		}
		public function fetch($path, $options = array()) 
		{
			$query = http_build_query(array_merge($options, array('version' => self::API_VERSION)));
			$request = curl_init('https://api.sipgate.net/my/'.implode("/", $path).'/?'.$query);
			
			curl_setopt_array($request, array(
				CURLOPT_USERPWD 		=> self::$username.":".self::$password,
				CURLOPT_SSL_VERIFYPEER 	=> false,
			    CURLOPT_HTTPHEADER 		=> array("Accept: application/json"),
			    CURLOPT_RETURNTRANSFER 	=> true
			));
					
			$result = curl_exec($request);
			$info 	= curl_getinfo($request);
					
			if ($info[http_code] == 401) 		trigger_error($info[http_code].': Not authorized to access the Sipgate API (check username and password)', E_USER_ERROR);
			else if ($info[http_code] !== 200) 	trigger_error($info[http_code].': Error during the Sipgate API request', E_USER_ERROR);
					
			$result = current(json_decode($result));
			curl_close($request);
					
			return $result;
		}
	}
	
	class Calls
	{
		public function __construct() 
		{
			echo "\n<b>new ".__CLASS__."()</b>\n";
			print_r(get_class_methods($this));
			
		}
		public function init($callee, $caller = '') {
			//echo parent::$username;
			//Main::fetch();
			print_r('Calling: '.$callee);
			return $this;
		}
	}
	
	class Contacts
	{
		public function __construct() 
		{
			echo "\n<b>new ".__CLASS__."()</b>\n";
			print_r(get_class_methods($this));
			
			$result = Main::fetch(array('contacts'), array('limit'=>0));
			
			$translate = array(
				'ADR' => 'address', 
				'FN' => 'alias', 
				'TEL' => 'phone', 
				'BDAY' => 'birthday', 
				'NOTE' => 'note', 
				'EMAIL' => 'email', 
				'URL' => 'url', 
				'ORG' => 'company', 
				'N' => 'name'
			);
			foreach($result->items as $item) {
				$contact = (object)null;
				foreach($item->entries as $subarray)
				{
					
					$name = $subarray->entryName ? $subarray->entryName : 'VAR';
					if(array_key_exists($name, $translate)) $name = $translate[$name];
					$name = strtolower($name);
					$type = strtolower($subarray->type);
					unset($subarray->entryName, $subarray->type);
	
					if($type) { $contact->$name->$type = (object)$subarray; } 
					else { $contact->$name = (object)$subarray; }	
				}
				$contact->id = $item->contactMeta->UUID;
				$contact->owner = $item->contactMeta->owner;
				$this->data[] = $contact;
			}
		}
		
		// public function __set($name, $value)
		// 	    {
		// 	        echo "Setting '$name' to '$value'\n";
		// 	        //$this->data[$name] = $value;
		// 	print_r($value);
		// 	    }

	    public function __get($name)
	    {
	        // echo "Getting '$name'\n";
	        // 			return $this->$nam
			
			
			
	        // if (array_key_exists($name, $this->data)) {
	        //     return $this->data->$name;
	        // }

	        // $trace = debug_backtrace();
	        // trigger_error(
	        //     'Undefined property via __get(): ' . $name .
	        //     ' in ' . $trace[0]['file'] .
	        //     ' on line ' . $trace[0]['line'],
	        //     E_USER_NOTICE);
	        // return null;
	    }
		
		public function search($options = array()) 
		{
			$this->data = array_filter($this->data, function ($c) use ($options) 
			{ 
				return ($c->id == $options['id']); 
			});
						
			// Resets the array => [0]
			$this->data = array_values($this->data);
			return $this;
		}
		public function call($options = array()) 
		{
			foreach($this->data[0]->entries as $item) 
			{
				if($item->entryName == 'TEL' && ($options['type'] ? $item->type == $options['type'] : true)) 
				{
					return Calls::init($item->value);
				}
			}
			return $this;
		}

	}
	
}

namespace 
{
	class Sipgate
	{
		public function __construct($username, $password) 
		{
			echo "\n<b>new ".__CLASS__."()</b>\n";
			print_r(get_class_methods($this));
			
			Sipgate\Main::login($username, $password);
		}
		
		public function calls()
		{
			return new Sipgate\Calls();
		}
		public function contacts()
		{
			return new Sipgate\Contacts();
		}
		

	}
}



 
 
// class Sipgate
// {		
// 	private static $username;
// 	private static $password;
// 	//const calls;
// 	
// 	public static function login($user, $pass) 
// 	{
// 		self::$username = $user;
// 		self::$password = $pass;
// 		//self::calls = new Calls();
// 	}
// 	
// 	public static function calls() 
// 	{
// 		return Calls;
// 	}
// 	
// 	
// 	
// 	
// 	protected static function fetch($credentials = array()) 
// 	{
// 		echo self::$password;
// 	}
// 	
// }
// 
// class Calls extends Sipgate
// {
// 	public function get($credentials = array()) 
// 	{
// 		echo "get";
// 		self::fetch();
// 	}
// }


// class Sipgate 
// {
// 	const API_VERSION = '2.37.0';
// 	
// 	public function __construct($credentials = array()) 
// 	{
// 		echo "\n<b>new ".__CLASS__."()</b>\n";
// 		print_r(get_class_methods($this));
// 		
// 		$this->credentials = (object) $credentials;
// 	}
// 	
// 
// 	
// 	// public function balance() {
// 	// 	return $this->fetch(array('billing', 'balance'));
// 	// }
// 	// 
// 	// public function events($options = array()) {
// 	// 	return $this->fetch(array('events'), $options);
// 	// }
// 	// 
// 	// public function contacts($options = array()) {
// 	// 	return $this->fetch(array('contacts'), $options);
// 	// }
// 	// 
// 	// public function contact($id, $options = array()) {
// 	// 	$contacts = $this->fetch(array('contacts'), $options);
// 	// 	$contact = array_filter($contacts->items, function ($c) use ($id) { return ($c->contactMeta->UUID == $id); });		
// 	// 	return $contact[0];
// 	// }
// 	// 
// 	
// 	private function fetch($path, $options = array()) 
// 	{
// 		$request = curl_init('https://api.sipgate.net/my/'.implode("/", $path).'/?'.http_build_query(array_merge($options, array('version' => self::API_VERSION))));
// 		
// 		curl_setopt($request, CURLOPT_USERPWD, $this->credentials->user.":".$this->credentials->password); 
// 		curl_setopt($request, CURLOPT_SSL_VERIFYPEER, false);
// 	    curl_setopt($request, CURLOPT_RETURNTRANSFER, true);
// 		curl_setopt($request,CURLOPT_HTTPHEADER,array("Accept: application/json"));
// 		
// 		$result = curl_exec($request);
// 		$info = curl_getinfo($request);
// 		
// 		if ($info[http_code] == 401) trigger_error($info[http_code].': Not authorized to access the Sipgate API (check username and password)', E_USER_ERROR);
// 		else if ($info[http_code] !== 200) trigger_error($info[http_code].': Error during the Sipgate API request', E_USER_ERROR);
// 		
// 		$result = current(json_decode($result));
// 		curl_close($request);
// 		
// 		return $result;
// 	}
// }
// 
// class Call extends Sipgate
// {
// 	public function __construct() 
// 	{
// 		echo "\n<b>new ".__CLASS__."()</b>\n";
// 		print_r(get_class_methods($this));
// 	}
// 	public function calls()
// 	{
// 		//return $this;
// 	}
// 	
// }


?>